import shutil
import sys
import os
import importlib.util
import pathlib
import requests

dbbpath = str(pathlib.Path(__file__).parent.resolve())
if not dbbpath in sys.path:
	sys.path.insert(0, dbbpath)

from prompt_helpers import (
	GithubJeex,
	Casting,
	Css,
	Timetools,
	ErrorLog,
	Ins,
	LocalUser,
)

class DbGlo:
	# globals for this app
	gloos = dict(
		_VERSION = '2026.11.001',
		_APP_NAME = 'check11',
		_DEV = False,
	)
	@classmethod
	def get(cls, s: str):
		return cls.gloos[s]

def check_version():
	githuburl = "https://raw.githubusercontent.com/jeex/jeex_public/refs/heads/main/check11_version.txt"
	try:
		r = requests.get(githuburl)
	except:
		sys.exit("Github not available")
	if r.status_code != 200:
		sys.exit("Github version.txt not available")
	dbb_version = r.content.decode("utf-8").strip()
	# print(dbb_version, DbGlo.get("_VERSION"))
	if dbb_version > DbGlo.get("_VERSION"):
		print(f"{Css.warning()}This version is {DbGlo.get('_VERSION')}. Upgrade with newer version {dbb_version} with: {Css.reset()}\n\tpip install check11 --upgrade --no-cache-dir")
		sys.exit(1)

def print_help():
	print(f"How to use check11 when your experiment is called {Css.bold()}my_exp.py{Css.reset()}: "
	      f"\n\t{Css.bold()}check11 -h{Css.reset()} for help"
	      f"\n\t{Css.bold()}check11 my_exp.py{Css.reset()} to run the test"
	      f"\n\t{Css.bold()}check11 my_exp.py -c{Css.reset()} to run the test, and clear terminal first"
	      f"\n\t{Css.bold()}check11 my_exp.py -v{Css.reset()} to run the test, with more info about errors"
	      f"\n\t{Css.bold()}deebecheck11ebros my_exp.py -t{Css.reset()} to run the test, with traceback for errors"
	      f"\n\t{Css.bold()}check11 my_exp.py -c -t -v{Css.reset()} to run the test, combining arguments"
      )

def from_args():
	email = experiment = None
	verbose = False
	clear = False
	trace = False
	phelp = False
	# results = False
	forcelog = False
	for arg in sys.argv[1:]:
		arg = arg.strip().lower()
		if arg in ['-v', '--verbose']:
			verbose = True
			continue
		if arg in ['-c', '--clear']:
			clear = True
			continue
		if arg in ['-t', '--trace']:
			trace = True
			continue
		if arg in ['-h', '--help']:
			phelp = True
			continue
		if arg in ['-l', '--forcelog']:
			forcelog = True
			continue
		if arg == '-version':
			print(DbGlo.get("_VERSION"))
			sys.exit(0)
		if arg.endswith('.py'):
			experiment = arg
			continue

	# show help
	if phelp:
		print_help()
		sys.exit(1)

	return experiment, verbose, trace, clear, forcelog

def download_unittest(check11pad, experiment):
	unittestname = get_unittest_name(experiment)
	print(f"Downloading unittest file for [{experiment}]...", end="")
	unittest_durl = f'https://raw.githubusercontent.com/jeex/jeex_public/refs/heads/main/check11_unittests/{unittestname}'
	if not GithubJeex().con_repo_download(unittest_durl, os.path.join(check11pad, unittestname)):
		print(f" failed")
		sys.exit(1)
	print(Css.good(" ready"))

def get_unittest_name(experiment):
	return f'check11_{experiment}'

def upload_test(experiment: str, check11pad: str):
	cwd = os.getcwd()
	source_path = os.path.join(cwd, experiment)
	target_path = os.path.join(check11pad, experiment)
	try:
		shutil.copy(source_path, target_path)
	except:
		print(Css.wrong(f"The file [{experiment}] does not exist in the directory [{cwd}]"))
		sys.exit(1)
	if not os.path.isfile(target_path):
		print(Css.wrong(f"The file [{experiment}] cannot be tested"))
		sys.exit(1)

def load_base_test_class(pad: str):
	try:
		# load basetest class
		specs = importlib.util.spec_from_file_location('base_test', os.path.join(pad, 'base_test.py'))
		# add to sys modules
		sys.modules['base_test'] = importlib.util.module_from_spec(specs)
		# load
		specs.loader.exec_module(sys.modules['base_test'])
	except Exception as e:
		print(Css.wrong("Base Test class is unavailable"))
		sys.exit(1)

def this_test_mod(modpath: str, modname: str):
	try:
		specs = importlib.util.spec_from_file_location(modname, modpath)
		sys.modules[modname] = importlib.util.module_from_spec(specs)
		specs.loader.exec_module(sys.modules[modname])
		return sys.modules[modname]
	except Exception as e:
		print(Css.wrong(f"The module [{modname}.py] cannot be loaded"))
		# print(e)
		sys.exit(1)

def clear_terminal():
	try:
		os.system('cls' if os.name == 'nt' else 'clear')
	except:
		pass

def testing_api_check(mon, report, percent, experiment):
	check = {
		'user': mon.gho.get_alias(),
		'experiment': experiment,
		'checked': Timetools.now_string(),
		'percent': percent,
		'report': report
	}
	print('update check result', mon.update_check(check))

def main():
	log = ErrorLog()
	check_version()
	check11_pad = os.path.join(os.path.dirname(__file__), 'unittests')
	if not os.path.isdir(check11_pad):
		try:
			os.makedirs(check11_pad)
		except Exception as e:
			log.add(Ins.info(), str(e))
			print(Css.wrong('No place for testing in your computer. Contact Monze.EU'))
			sys.exit(1)

	# read all stuff from command line args
	experiment, verbose, trace, clear, forcelog = from_args()
	log.set_force(forcelog)

	if clear:
		clear_terminal()

	# check experiment
	if experiment is None:
		print(f"{Css.wrong()}No valid experiment in command line args. Use: {Css.reset()}check11 -h{Css.wrong()} for help{Css.reset()}")
		print_help()
		sys.exit(1)
	cwd = os.getcwd()
	if not os.path.isfile(os.path.join(cwd, experiment)):
		print(Css.wrong(f"Invalid experiment name: [{experiment}] does not exist in path: {os.getcwd()}"))
		sys.exit(1)

	# download unittest, exits if wrong
	download_unittest(check11_pad, experiment)

	# upload the script experiment to unittests folder
	upload_test(experiment, check11_pad)

	# load test mod
	load_base_test_class(os.path.dirname(__file__))
	experiment_path = os.path.join(check11_pad, experiment)
	unittest_path = os.path.join(check11_pad, get_unittest_name(experiment))
	unittest_mod = this_test_mod(unittest_path, get_unittest_name(experiment))

	# start test class
	tm = unittest_mod.RunTest(
		experiment_path,
		experiment,
		verbose,
		trace,
	)
	# print results
	report = tm.get_report()
	percent = tm.get_score_percent()
	# print the report in the terminal
	print(tm)
	print()

if __name__ == "__main__":
	main()


import pickle
from datetime import datetime
import pytz
import time
import urllib.request
import certifi
import ssl
import requests
from colorama import Style, Fore, Back
import os
from inspect import currentframe, getframeinfo
import platformdirs

class Ins:
	@classmethod
	def info(cls):
		return f"[{cls.script()}|{cls.func()}|{cls.line()}]"

	@classmethod
	def script(cls):
		try:
			return getframeinfo(currentframe()).filename
		except:
			return '-'

	@classmethod
	def func(cls):
		try:
			return currentframe().f_code.co_name
		except:
			return '-'

	@classmethod
	def line(cls):
		try:
			return getframeinfo(currentframe()).lineno
		except:
			return '-'

class ErrorLogMeta(type):
	_instances = {}
	def __call__(cls, *args, **kwargs):
		if cls not in cls._instances:
			instance = super().__call__(*args, **kwargs)
			cls._instances[cls] = instance
		return cls._instances[cls]

class ErrorLog(metaclass=ErrorLogMeta):
	force = False

	def __init__(self):
		self.path = os.path.join(os.getcwd(), 'check11.log')
		self.add('start log', Timetools.now_string(), first=True)

	def __del__(self):
		try:
			self.close_log()
		except:
			pass

	def set_force(self, force):
		self.force = force

	def close_log(self):
		self.add('end log', Timetools.now_string())

	def set_path(self, path):
		self.path = path

	def add(self, where: str, msg: str, first=False):
		# self.msgs.append([where, msg])
		if first:
			mode = 'w'
		else:
			mode = 'a'
		if self.force:
			with open(self.path, mode=mode) as handle:
				handle.write(f"{where}\t{msg}\n")

class Css:
	# uses colorama for creating style in prompt
	@classmethod
	def reset(cls) -> str:
		return f"{Style.RESET_ALL}"

	@classmethod
	def normal(cls, s=None) -> str:
		if s is None:
			return f"{Style.NORMAL}"
		return f"{cls.normal()}{s}{cls.reset()}"

	@classmethod
	def href(cls, uri, label=None):
		if label is None:
			label = uri
		parameters = ''
		# OSC 8 ; params ; URI ST <name> OSC 8 ;; ST
		escape_mask = '\033]8;{};{}\033\\{}\033]8;;\033\\'
		return escape_mask.format(parameters, uri, label)

	@classmethod
	def jippie(cls, s=None) -> str:
		if s is None:
			return f"{Fore.BLUE}{Style.BRIGHT}"
		else:
			return f"{cls.jippie()}{s}{cls.reset()}"

	@classmethod
	def bold(cls, s=None) -> str:
		if s is None:
			return f"{Style.BRIGHT}"
		return f"{cls.bold()}{s}{cls.reset()}"

	@classmethod
	def good(cls, s=None) -> str:
		if s is None:
			return f"{Fore.LIGHTGREEN_EX}{Style.BRIGHT}"
		return f"{cls.good()}{s}{cls.reset()}"

	@classmethod
	def warning(cls, s=None) -> str:
		if s is None:
			return f"{Fore.LIGHTCYAN_EX}{Style.BRIGHT}"
		return f"{cls.warning()}{s}{cls.reset()}"

	@classmethod
	def attention(cls, s=None) -> str:
		# more serious than warn
		if s is None:
			return f"{Back.LIGHTYELLOW_EX}{Fore.BLACK}{Style.BRIGHT}"
		return f"{cls.attention()} {s} {cls.reset()}"

	@classmethod
	def wrong(cls, s=None) -> str:
		if s is None:
			return f"{Fore.LIGHTRED_EX}{Style.BRIGHT}"
		return f"{cls.wrong()}{s}{cls.reset()}"

	@classmethod
	def prompt(cls, s=None) -> str:
		if s is None:
			return f"{Fore.BLACK}{Style.BRIGHT}"
		return f"{cls.prompt()}{s}{cls.reset()}"

	@classmethod
	def log(cls) -> str:
		return f"{Fore.MAGENTA}{Style.NORMAL}"

# General function for type casting
class Casting:
	@classmethod
	def str_(cls, erin, default: str | None='') -> str | None:
		try:
			return str(erin)
		except:
			return default

	@classmethod
	def int_(cls, erin, default: int | None=0) -> int | None:
		try:
			return int(erin)
		except:
			return default

	@classmethod
	def float_(cls, erin, default=0.0) -> float:
		try:
			return float(erin)
		except:
			return default

	@classmethod
	def bool_(cls, erin, default=True) -> bool:
		try:
			return bool(erin)
		except:
			return default

	@classmethod
	def listint_(cls, erin, default=[]):
		try:
			for i in range(len(erin)):
				erin[i] = int(erin[i])
			return erin
		except:
			return default

	@classmethod
	def liststr_(cls, erin, default=[]):
		try:
			for i in range(len(erin)):
				erin[i] = str(erin[i])
			return erin
		except:
			return default

	@classmethod
	def cast(cls, erin, intotype, default=None) -> any:
		if intotype == int:
			if default is None:
				return cls.int_(erin)
			else:
				return cls.int_(erin, default=default)
		elif intotype == float:
			if default is None:
				return cls.float_(erin)
			else:
				return cls.float_(erin, default=default)
		elif intotype == bool:
			if default is None:
				return cls.bool_(erin)
			else:
				return cls.bool_(erin, default=default)
		return str(erin).strip()

	@classmethod
	def typecast_list(cls, l: list, t: type) -> list:
		try:
			return list(map(t, l))
		except Exception as e:
			return []

# General functions for working with time
class Timetools:
	TTIMESTRING = "%Y%m%dT%H00"
	DATETIME_LOCAL = "%Y-%m-%dT%H:%M"
	DATETIMESTRING = "%Y-%m-%d %H:%M:%S"
	DATETIMESTRING_NL = "%d-%m-%Y %H:%M:%S"
	DATESTRING = "%Y-%m-%d"
	DATESTRING_NL = "%d-%m-%Y"
	BIRTH = '1972-02-29'

	@classmethod
	def dtlocal_2_ts(cls, tts: str):
		try:
			dt = datetime(
				year=int(tts[0:4]),
				month=int(tts[5:7]),
				day=int(tts[8:10]),
				hour=int(tts[11:13]),
				minute=int(tts[14:16])
			)
			return int(dt.timestamp())
		except:
			return Timetools.td_2_ts(cls.BIRTH)

	@classmethod
	def dtonixzips_2_tms(cls, tts: str):
		try:
			dt = datetime(
				year=int(tts[0:4]),
				month=int(tts[5:7]),
				day=int(tts[8:10]),
				hour=int(tts[11:13]),
				minute=int(tts[14:16]),
				second=int(tts[17:19]),
				microsecond=int(tts[20:])
			)
			return int(dt.timestamp() * 1000)
		except Exception as e:
			return Timetools.td_2_ts(cls.BIRTH) * 1000

	@classmethod
	def td_2_ts(cls, datum: str) -> int:
		# convert date-string yyyy-mm-dd to seconds timestamp
		try:
			dt = datetime(
				year=int(datum[0:4]),
				month=int(datum[5:7]),
				day=int(datum[8:10]),
			)
			return int(dt.timestamp())
		except:
			return Timetools.td_2_ts(cls.BIRTH)

	@classmethod
	def tdtime_2_ts(cls, datumtijd: str) -> int:
		# convert date-string yyyy-mm-dd to seconds timestamp
		try:
			dt = datetime(
				year=int(datumtijd[0:4]),
				month=int(datumtijd[5:7]),
				day=int(datumtijd[8:10]),
				hour=int(datumtijd[11:13]),
				minute=int(datumtijd[14:16]),
				second=int(datumtijd[17:19]),
			)
			return int(dt.timestamp())
		except:
			return Timetools.td_2_ts(cls.BIRTH)

	@classmethod
	def ts_2_td(cls, timest: int, rev=False, withtime=False) -> str:
		# convert seconds to datestring yyyy-mm-dd
		if withtime:
			if rev:
				dstr = cls.DATETIMESTRING
			else:
				dstr = cls.DATETIMESTRING_NL
		else:
			if rev:
				dstr = cls.DATESTRING
			else:
				dstr = cls.DATESTRING_NL
		try:
			return datetime.fromtimestamp(timest, pytz.timezone("Europe/Amsterdam")).strftime(dstr)
		except:
			return ''

	@classmethod
	def now(cls) -> float:
		return time.time()

	@classmethod
	def now_secs(cls) -> int:
		# for normal use
		return int(cls.now())

	@classmethod
	def now_milisecs(cls) -> int:
		# for use in generating unique numbers
		return int(cls.now() * 1000)

	@classmethod
	def now_nanosecs(cls) -> int:
		# not preferred
		return int(cls.now() * 1000000)

	@classmethod
	def ts_2_datetimestring(cls, ts: int|float|None, rev=False, noseconds=False):
		if rev:
			dstr = cls.DATETIMESTRING
		else:
			dstr = cls.DATETIMESTRING_NL
		if noseconds:
			dstr = dstr[:-3]
		if ts is None:
			ts = cls.now()
		if isinstance(ts, int):
			if len(str(ts)) > 11:
				ts = ts / 1000 # nanoseconds
		if not isinstance(ts, float):
			ts = Casting.float_(ts, 0) # adding trailing zero's representing ms and ns
		return datetime.fromtimestamp(ts, pytz.timezone("Europe/Amsterdam")).strftime(dstr)

	@classmethod
	def ts_2_datestring(cls, ts: int | float | None, rev=False):
		if rev:
			dstr = cls.DATESTRING
		else:
			dstr = cls.DATESTRING_NL

		if ts is None:
			ts = cls.now()
		if isinstance(ts, int):
			if len(str(ts)) > 13:
				ts = ts / 1000000  # nanoseconds
			elif len(str(ts)) > 11:
				ts = ts / 1000  # milliseconds
		if not isinstance(ts, float):
			ts = Casting.float_(ts, 0)  # adding trailing zero's representing ms and ns
		return datetime.fromtimestamp(ts, pytz.timezone("Europe/Amsterdam")).strftime(dstr)

	@classmethod
	def now_string(cls) -> str:
		return datetime.fromtimestamp(cls.now(), pytz.timezone("Europe/Amsterdam")).strftime(cls.DATETIMESTRING)
		# return str(datetime.strptime(timestamp, cls.DATETIMESTRING))

	@classmethod
	def datetimenow(cls):
		return datetime.now()

	@classmethod
	def draaiom(cls, erin):
		# changes yyyy-mm-dd into dd-mm-yyyy and vv
		try:
			d = erin.split('-')
			return f'{d[2]}-{d[1]}-{d[0]}'
		except:
			return erin

class GithubJeex:
	@staticmethod
	def con_repo_list():
		pass

	@staticmethod
	def con_repo_path(token, assignment) -> dict|None:
		log = ErrorLog()
		# gets all the info about a repo
		data = data = {
			"Accept": "application/vnd.github+json",
			"Authorization": f"Bearer {token}",
			"X-GitHub-Api-Version": "2022-11-28",
		}
		url = f'https://api.github.com/repos/jeex/jeex_public/contents/{assignment}'
		try:
			response = requests.get(
				url,
				data
			)
		except Exception as e:
			log.add(Ins.info(), str(e))
			return None
		try:
			return response.json()
		except Exception as e:
			log.add(Ins.info(), str(e))
			return None

	@staticmethod
	def con_repo_download(url, fpath) -> bool:
		log = ErrorLog()
		try:
			with urllib.request.urlopen(url, context=ssl.create_default_context(cafile=certifi.where())) as handle:
				script = handle.read().decode()
			with open(fpath, 'w') as handle:
				handle.write(script)
			return True
		except Exception as e:
			log.add(Ins.info(), str(e))
			return False
		
class LocalUser:
	_expire = 1000 * 60 * 60 * 48 # twee dagen
	def __init__(self):
		# self._expire = 1000 * 60
		pass

	def expired(self, toens) -> bool:
		nus = Timetools.now_milisecs()
		return toens + self._expire < nus

	# keeps user data locally
	def get_path(self):
		path = platformdirs.user_data_dir("monze.eu")
		if not os.path.isdir(path):
			os.makedirs(path)
		return os.path.join(path, "check11_settings.pickle")

	def set(self, d: dict) -> bool:
		# sets user in local settings file
		nus = Timetools.now_milisecs()
		d['nus'] = nus
		try:
			with open(self.get_path(), "wb") as handle:
				pickle.dump(d, handle)
			return True
		except:
			pass
		return False

	def get(self) -> dict | None:
		# gets user from local settings file
		nus = Timetools.now_milisecs()
		try:
			with open(self.get_path(), "rb") as handle:
				d = pickle.load(handle)
			# check if expired
			if self.expired(d['nus']):
				self.remove()
				return None
			return d
		except:
			pass
		return None

	def remove(self):
		try:
			os.remove(self.get_path())
		except:
			pass

